package bin;

import lib.CLIAppBase;
import kernel.peripherals.Peripherals.Peripheral;
import kernel.net.Routing;

using tink.CoreApi;

class Net extends CLIAppBase {
	public function new() {
		registerSyncSubcommand("route", (args)->{
			var routes = Routing.instance.getRouteTable();

			for(k => v in routes) {
				handle.writeLine('${k} => ${v.interf.name()}(${v.cost})');
			}

			return true;
		});

		registerSyncSubcommand("iface", (args)->{
			var modems = Peripheral.instance.getAllModems();

			for (modem in modems) {
				handle.writeLine(modem.name());
			}

			return true;
		});

		registerSyncSubcommand("proto",(args)->{
			var protos = kernel.net.Net.instance.getActiveProtocols();

			for (proto in protos) {
				handle.writeLine(proto);
			}

			return true;
		});

		registerAsyncSubcommand("ping",(args)->{
			if (args.length < 1) {
				return Future.sync(false);
			}
	
			var toID:Null<Int> = Std.parseInt(args[0]);
	
			if (toID == null) {
				handle.write("Invalid ID");
				return Future.sync(false);
			}
	
			return kernel.net.Net.instance.ping(toID).map(result -> {
				switch (result){
					case Success(_):
						handle.write("Ping succeeded");
					case Failure(failure):
						handle.write("Ping failed: " + failure);
				}

				return true;
			});
		},"<id>");
	}
}
